from e3.fs import cp
from e3.testsuite.result import TestStatus
from drivers import gprbuild, GNATcollTestDriver
from drivers.valgrind import check_call_valgrind
import os


class BasicTestDriver(GNATcollTestDriver):
    """Default GNATcoll testsuite driver.

    In order to declare a test:

    1- Create a directory with a test.yaml inside
    2- Add test sources in that directory
    3- Add a main called test.adb that use support/test_assert.ads package.
    4- Do not put test.gpr there, it breaks the test, if you need a project
       file for testing, name it something else.
    5- If you need additional files for you test, list them in test.yaml:
       data:
           - "your_file1"
           - "your_file2"
    """

    def add_test(self, dag):
        """Declare test workflow.

        The workflow is the following::

            build --> check status

        :param dag: tree of test fragment to amend
        :type dag: e3.collection.dag.DAG
        """
        self.add_fragment(dag, 'build')
        self.add_fragment(dag, 'check_run', after=['build'])

        if 'test_exe' not in self.test_env:
            self.test_env['test_exe'] = 'obj/test'

    def build(self, previous_values):
        """Build fragment."""
        skip = self.should_skip()
        if skip is not None:
            self.result.set_status(skip)
            self.push_result()
            return False

        if self.test_env.get('no-coverage'):
            gpr_project_path = self.env.gnatcoll_prod_gpr_dir
        else:
            gpr_project_path = self.env.gnatcoll_gpr_dir
        return gprbuild(self, gcov=self.env.gcov,
                        gpr_project_path=gpr_project_path)

    def check_run(self, previous_values):
        """Check status fragment."""
        if not previous_values['build']:
            return

        for data in self.test_env.get('data', []):
            cp(os.path.join(self.test_env['test_dir'], data),
               self.test_env['working_dir'], recursive=True)

        process = check_call_valgrind(
            self,
            [os.path.join(self.test_env['working_dir'],
                          self.test_env['test_exe'])])
        if '<=== TEST PASSED ===>' not in process.out:
            self.result.set_status(TestStatus.FAIL)
        else:
            self.result.set_status(TestStatus.PASS)
        self.push_result()



# -*- coding: utf-8 -*-

# GNATcoll Bindings - Readline documentation build configuration file

# General information about the project.
project = u'GNATcoll Bindings - Readline'

# Output file base name for HTML help builder.
htmlhelp_basename = 'GNATcoll-Readline'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
  ('index', 'GNATcoll-Readline.tex',
   u'GNATcoll Bindings - Readline Documentation', u'AdaCore', 'manual'),
]

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'gnatcoll-readline',
     u'GNATcoll Bindings - Readline Documentation',
     [u'AdaCore'], 1)
]

# Bibliographic Dublin Core info.
epub_title = u'GNATcoll Bindings - Readline'

execfile('../../docs-common/common_conf.py')



from e3.testsuite.process import check_call


def check_call_valgrind(driver, cmd, test_name=None, result=None, **kwargs):
    """
    Wrapper for `e3.testsuite.process` that runs the process under Valgrind if
    this is a Valgrind-checked testsuite run. The process exit status will be
    2 if Valgrind finds memory issues.
    """
    if driver.env.valgrind:
        cmd = ['valgrind', '-q', '--error-exitcode=2',
               '--leak-check=full'] + cmd
    return check_call(driver, cmd, test_name, result, **kwargs)



#!/usr/bin/env python

import os
import os.path
import re

pkg_re = re.compile("^(private)?\s*package\s*(\S+)")


def recursive_ls(dir):
    """Return the list of ads files in dir and its subdirs"""
    result = set()
    for f in os.listdir(dir):
        if f.endswith(".ads") \
           and f.startswith("gnatcoll-"):

            private = False
            pkg = ""
            for l in file(os.path.join(dir, f)).readlines():
                m = pkg_re.search(l)
                if m:
                    private = m.group(1)
                    pkg = m.group(2)
                    break

            if not private:
                result.add((pkg, os.path.splitext(f)[0]))

        elif os.path.isdir(os.path.join(dir, f)):
            result = result.union(recursive_ls(os.path.join(dir, f)))

    return result

list = recursive_ls("..")
out = file("help_gnatcoll-bindings.py", "wb")
out.write("""XML = r'''<?xml version="1.0"?>
<GPS>
""")

for pkg, f in sorted(list):
    if '__' in f:
        # An internal package with a specific naming scheme
        continue

    menu = pkg.replace(".", "/").replace("_", "__")

    # Do we have a submenu ?
    in_front = False
    for pkg2, b in list:
        if b.startswith(f + "-"):
            item = menu[menu.rfind("/") + 1:]
            menu = menu + "/&lt;" + item + "&gt;"
            break

    out.write("""  <documentation_file>
     <shell>Editor.edit "%(file)s.ads"</shell>
     <descr>%(package)s</descr>
     <menu>/Help/%(menu)s</menu>
     <category>GNAT Components Collection</category>
  </documentation_file>

""" % {"file": f, "menu": menu, "package": pkg})

out.write("""</GPS>'''
import GPS
GPS.parse_xml(XML)
""")
out.close()



from e3.fs import rm
from e3.testsuite.driver import TestDriver
from e3.testsuite.result import TestStatus, TestResult
from drivers import gprbuild
from drivers.valgrind import check_call_valgrind
import os


class DataValidationDriver(TestDriver):
    """Data validation driver.

    For each test program call the program with data file defined in
    data_files key of the test. If the program returns 0 assume that
    the test passed.
    """

    def add_test(self, dag):
        self.add_fragment(dag, 'build')

        tear_down_deps = []
        for data_file, description in self.test_env['data_files'].iteritems():
            tear_down_deps.append(data_file)
            self.add_fragment(
                dag,
                data_file,
                fun=lambda x, d=data_file, m=description:
                self.run_subtest(d, m, x),
                after=['build'])
        self.add_fragment(dag, 'tear_down', after=tear_down_deps)

    def run_subtest(self, data_file, description, previous_values):
        test_name = self.test_name + '.' + data_file
        result = TestResult(test_name, env=self.test_env)

        if not previous_values['build']:
            return TestStatus.FAIL

        process = check_call_valgrind(
            self,
            [os.path.join(self.test_env['working_dir'],
                          self.test_env.get('validator', 'obj/test')),
             os.path.join(self.test_env['test_dir'], data_file)],
            result=result)

        return self.validate_result(process, data_file, result)

    def validate_result(self, process, data_file, result):
        # Read data file
        if '<=== TEST PASSED ===>' in process.out:
            return TestStatus.PASS
        else:
            result.set_status(TestStatus.FAIL)
            self.push_result(result)
            return TestStatus.FAIL

    def tear_down(self, previous_values):
        failures = [v for v in previous_values.values() if
                    not isinstance(v, TestStatus) or v != TestStatus.PASS]
        if failures:
            self.result.set_status(TestStatus.FAIL,
                                   msg="%s subtests failed" % len(failures))
        else:
            self.result.set_status(TestStatus.PASS)

        self.push_result()

        if self.env.enable_cleanup:
            rm(self.test_env['working_dir'], recursive=True)

    def build(self, previous_values):
        return gprbuild(self, gcov=self.env.gcov,
                        gpr_project_path=self.env.gnatcoll_gpr_dir)



from Test import My_Class

# pass an integer in the field waiting for a float
m = My_Class(42)
# the returned value should be a float
assert m.get_value() == 42.0
# Computation
m.set_value(m.get_value() / 2 + 0.1)
# Verify the value almost match
assert str(m.get_value()).startswith("21.1")
print('<=== TEST PASSED ===>')



#!/usr/bin/env python
import logging
import sys
import re
import os
import json
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from setup_support import SetupApp

PYTHON_DATA_SCRIPT = """
from distutils.sysconfig import (get_config_var, get_python_inc,
                                 get_config_vars, PREFIX)
import json
result = {'config_vars': get_config_vars(),
          'python_inc': get_python_inc(),
          'python_inc_plat': get_python_inc(plat_specific=True),
          'prefix': PREFIX}
print(json.dumps(result))
"""


def fetch_python_config(config):
    logging.info('Fetch Python information...')
    python_output = config.run(config.data['python_exec'], '-c',
                               PYTHON_DATA_SCRIPT,
                               grab=True)
    python_data = json.loads(python_output)
    config_vars = python_data['config_vars']
    python_version = config_vars["VERSION"]
    python_ldversion = config_vars.get("LDVERSION", python_version)
    logging.info('  %-24s %s', 'Python version:', python_version)

    # Current python location
    current_prefix = python_data['prefix']

    # Fetch prefix during the build process. Some paths of interest might
    # still reference a location used during the Python build process.
    build_prefix = ([sys.prefix] +
                    re.findall(r"'--prefix=([^']+)'",
                               config_vars.get('CONFIG_ARGS', '')))[-1]

    def relocate(path):
        if os.path.isabs(path):
            rel_path = os.path.relpath(path, build_prefix)
            if not rel_path.startswith(os.pardir):
                # If the input path is relative to the original build
                # directory, replace build prefix by the current one.
                return os.path.join(current_prefix, rel_path)
            else:
                # Otherwise, return it unchanged
                return path
        else:
            # The input path is relative so assume it's relative to the current
            # python prefix.
            return os.path.join(current_prefix, path)

    # Retrieve cflags, and linker flags
    static_dir = relocate(config_vars.get('LIBPL', 'libs'))
    logging.info('  %-24s %s', 'Static dir', static_dir)

    shared_dir = relocate(config_vars.get('LIBDIR', '.'))
    logging.info('  %-24s %s', 'Shared dir', shared_dir)

    # Add first relocated include dirs followed by non-relocated version.
    # Indeed, when using venv and maybe virtualenv, includes are not copied.
    cflags = "-I" + relocate(python_data['python_inc'])
    if python_data['python_inc'] != python_data['python_inc_plat']:
        cflags += " -I" + relocate(python_data['python_inc_plat'])
        cflags += " -I" + python_data['python_inc_plat']
    cflags += " -I" + python_data['python_inc']

    logging.info('  %-24s %s', 'CFLAGS', cflags)

    if python_version.startswith('3'):
        # In python 3.x MODLIBS seems to drag in too many libraries
        python_libs = [config_vars[v] for v in ("LIBS", "SYSLIBS")
                       if v in config_vars and config_vars[v]]
    else:
        python_libs = [config_vars[v] for v in ("LIBS", "SYSLIBS", "MODLIBS")
                       if v in config_vars and config_vars[v]]
    python_libs = " ".join(python_libs)

    python_shared_libs = "-L%s -lpython%s %s" % (shared_dir,
                                                 python_ldversion,
                                                 python_libs)
    python_static_libs = ''
    libpython_a = os.path.join(
        static_dir,
        config_vars.get('LIBRARY', 'libpython%s.a' % python_version))

    if os.path.isfile(libpython_a):
        python_static_libs = libpython_a + ' ' + python_libs
    if sys.platform.startswith('linux'):
        # On Linux platform, even when linking with the static libpython,
        # symbols not used by the application itself should be exported so
        # that shared library present in Python can use the Python C API.
        python_static_libs += ' -export-dynamic'
        python_shared_libs += ' -export-dynamic'

    logging.info('  %-24s %s', 'Shared linker flags', python_shared_libs)
    logging.info('  %-24s %s', 'Static linker flags', python_static_libs)

    # User does not have the choice between linking with static libpython
    # and shared libpython. If --enable-shared or --enable-framework was
    # passed to Python's configure during Python build, then we should
    # link with the shared libpython, otherwise with the static one.
    # Indeed otherwise some C modules might not work as expected or even
    # crash. On Windows always link with shared version of libpython
    # (if the static is present, this is just an indirection to the shared)
    if '--enable-shared' in config_vars.get('CONFIG_ARGS', '') or \
            '--enable-framework' in config_vars.get('CONFIG_ARGS', '') or \
            sys.platform.startswith('win'):
        logging.info('Force link to shared python library')
        config.set_data('GNATCOLL_PYTHON_LIBS',
                        python_shared_libs, sub='gprbuild')
        config.set_data('GNATCOLL_LIBPYTHON_KIND', 'shared',
                        sub='gprbuild')
    else:
        logging.info('Force link to static python library')
        config.set_data('GNATCOLL_PYTHON_LIBS',
                        python_static_libs, sub='gprbuild')
        config.set_data('GNATCOLL_LIBPYTHON_KIND', 'static',
                        sub='gprbuild')
    config.set_data('GNATCOLL_PYTHON_CFLAGS', cflags, sub='gprbuild')


class GNATCollPython(SetupApp):
    name = 'gnatcoll_python'
    project = 'gnatcoll_python.gpr'
    description = 'GNATColl Python bindings'

    def create(self):
        super(GNATCollPython, self).create()
        self.build_cmd.add_argument(
            '--python-exec',
            help='set python executable location',
            metavar='PATH',
            default=sys.executable)
        self.build_cmd.add_argument(
            '--debug',
            help='build project in debug mode',
            action="store_true",
            default=False)

    def update_config(self, config, args):
        # Fetch python information
        config.set_data('python_exec', args.python_exec)
        fetch_python_config(config)

        logging.info('%-26s %s',
                     'Libraries kind', ", ".join(config.data['library_types']))

        # Set library version
        with open(os.path.join(config.source_dir, '..',
                               'version_information'), 'r') as fd:
            version = fd.read().strip()
        config.set_data('GNATCOLL_VERSION', version, sub='gprbuild')
        logging.info('%-26s %s', 'Version', version)

        # Set build mode
        config.set_data('BUILD', 'DEBUG' if args.debug else 'PROD',
                        sub='gprbuild')
        logging.info('%-26s %s', 'Build mode',
                     config.data['gprbuild']['BUILD'])

        # Set GNATCOLL_OS
        if 'darwin' in config.data['canonical_target']:
            gnatcoll_os = 'osx'
        elif 'windows' in config.data['canonical_target']:
            gnatcoll_os = 'windows'
        else:
            # Assume this is an Unix system
            gnatcoll_os = 'unix'
        config.set_data('GNATCOLL_OS', gnatcoll_os, sub='gprbuild')

    def variants(self, config, cmd):
        result = []
        for library_type in config.data['library_types']:
            gpr_vars = {'LIBRARY_TYPE': library_type,
                        'XMLADA_BUILD': library_type,
                        'GPR_BUILD': library_type}
            if cmd == 'install':
                result.append((['--build-name=%s' % library_type,
                                '--build-var=LIBRARY_TYPE'],
                               gpr_vars))
            else:
                result.append(([], gpr_vars))
        return result


if __name__ == '__main__':
    app = GNATCollPython()
    sys.exit(app.run())



from e3.fs import mkdir
from e3.os.process import Run
from e3.os.fs import df
from e3.testsuite.driver import TestDriver
from e3.testsuite.process import check_call
from e3.testsuite.result import TestStatus
import os
import logging
import traceback
import sys


# Root directory of respectively the testsuite and the gnatcoll
# repository.
TESTSUITE_ROOT_DIR = os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))
GNATCOLL_ROOT_DIR = os.path.dirname(TESTSUITE_ROOT_DIR)

PROJECT = {
    'omp': 'gnatcoll_omp.gpr',
    'gmp': 'gnatcoll_gmp.gpr',
    'iconv': 'gnatcoll_iconv.gpr'
}


def make_gnatcoll(work_dir, gcov=False):
    """Build gnatcoll core with or without gcov instrumentation.

    :param work_dir: working directory. gnatcoll is built in `build` subdir
        and installed in `install` subdir
    :type work_dir: str
    :param gcov: if False then build gcov in PROD mode, otherwise
        build it with gcov instrumentation in DEBUG mode
    :type gcov: bool
    :return: a triplet (project path, source path, object path)
    :rtype: (str, str, str)
    :raise AssertError: in case compilation of installation fails
    """

    # Create build tree structure
    build_dir = os.path.join(work_dir, 'build')
    install_dir = os.path.join(work_dir, 'install')
    mkdir(build_dir)
    mkdir(install_dir)

    # Compute make invocation
    for binding in ('gmp', 'omp', 'iconv', 'python'):
        logging.info('Compiling gnatcoll %s (gcov=%s)', binding, gcov)
        setup = os.path.join(GNATCOLL_ROOT_DIR, binding, 'setup.py')
        obj_dir = os.path.join(build_dir, binding)
        mkdir(obj_dir)

        build_cmd = [sys.executable, setup, 'build', '--library-types=static']
        install_cmd = [sys.executable, setup, 'install',
                       '--prefix', install_dir]

        if gcov:
            build_cmd += ['--gpr-opts', '-cargs', '-fprofile-arcs',
                          '-ftest-coverage',
                          '-largs', '-lgcov',
                          '-gargs', '-XBUILD=DEBUG']
        else:
            build_cmd += ['--gpr-opts', '-XBUILD=PROD']

        # Build & Install
        p = Run(build_cmd, cwd=obj_dir)
        assert p.status == 0, \
            "gnatcoll %s build failed:\n%s" % (binding, p.out)
        logging.debug('build:\n%s', p.out)

        p = Run(install_cmd, cwd=obj_dir)
        assert p.status == 0, \
            "gnatcoll %s installation failed:\n%s" % (binding, p.out)
        logging.debug('install:\n%s', p.out)

    return (os.path.join(install_dir, 'share', 'gpr'),
            os.path.join(install_dir, 'include'),
            build_dir)


def gprbuild(driver,
             project_file=None,
             cwd=None,
             gcov=False,
             scenario=None,
             gpr_project_path=None,
             **kwargs):
    """Launch gprbuild.

    :param project_file: project file to compile. If None, we looks first for
        a test.gpr in the test dir and otherwise fallback on the common
        test.gpr project of the support subdir of the testsuite.
    :type project_file: str
    :param cwd: directory in which to run gprbuild. If None the gprbuild build
        is run in the default working dir for the test.
    :type cwd: str | None
    :param gcov: if True link with gcov libraries
    :type gcov: bool
    :param scenario: scenario variable values
    :type scenario: dict
    :param gpr_project_path: if not None prepent this value to GPR_PROJECT_PATH
    :type gpr_project_path: None | str
    :param kwargs: additional keyword arguements are passed to
        e3.testsuite.process.check_call function
    :return: True on successful completion
    :rtype: bool
    """
    if scenario is None:
        scenario = {}

    if cwd is None:
        cwd = driver.test_env['working_dir']
    mkdir(cwd)

    if project_file is None:
        project_file = os.path.join(driver.test_env['test_dir'],
                                    'test.gpr')
        if not os.path.isfile(project_file):
            project_file = os.path.join(cwd, 'test.gpr')
            with open(os.path.join(TESTSUITE_ROOT_DIR, 'support',
                                   'test.gpr'), 'r') as fd:
                content = fd.read()
            with open(project_file, 'w') as fd:
                for component in driver.test_env.get('components', []):
                    fd.write('with "%s";\n' % PROJECT[component])
                fd.write(content)
            scenario['TEST_SOURCES'] = driver.test_env['test_dir']
    scenario['SUPPORT_SOURCES'] = os.path.join(TESTSUITE_ROOT_DIR, 'support')

    gprbuild_cmd = [
        'gprbuild', '--relocate-build-tree', '-p', '-P', project_file]
    for k, v in scenario.iteritems():
        gprbuild_cmd.append('-X%s=%s' % (k, v))
    if gcov:
        gprbuild_cmd += ['-largs', '-lgcov', '-cargs',
                         '-fprofile-arcs', '-ftest-coverage', '-g']

    # Adjust process environment
    env = None
    if gpr_project_path:
        new_gpr_path = gpr_project_path
        if 'GPR_PROJECT_PATH' in os.environ:
            new_gpr_path += os.path.pathsep + os.environ['GPR_PROJECT_PATH']
        env = {'GPR_PROJECT_PATH': new_gpr_path}

    check_call(
        driver,
        gprbuild_cmd,
        cwd=cwd,
        env=env,
        ignore_environ=False,
        **kwargs)
    # If we get there it means the build succeeded.
    return True


class GNATcollTestDriver(TestDriver):
    """Abstract class to share some common facilities."""

    def should_skip(self):
        """Handle of 'skip' in test.yaml.

        :return: None if the test should not be skipped, a TestStatus
            otherwise.
        :rtype: None | TestStatus
        """
        if 'skip' in self.test_env:
            eval_env = {
                'env': self.env,
                'test_env': self.test_env,
                'disk_space': lambda: df(self.env.working_dir)}

            for status, expr in self.test_env['skip']:
                try:
                    if eval(expr, eval_env):
                        return TestStatus[status]
                except Exception:
                    logging.error(traceback.format_exc())
                    return TestStatus.ERROR
        return None



from e3.testsuite.result import TestStatus
from drivers.data_validation import DataValidationDriver
import os
import json
import logging


class JSONValidationDriver(DataValidationDriver):

    def validate_result(self, process, data_file, result):
        # Read data file
        with open(os.path.join(self.test_env['test_dir'], data_file)) as fd:
            expected = json.load(fd)

        got = json.loads(process.out)
        if got != expected:
            logging.debug('%s\n<=>\n%s', got, expected)
            result.set_status(TestStatus.FAIL)
            self.push_result(result)
            return TestStatus.FAIL
        else:
            return TestStatus.PASS



#!/usr/bin/env python
import logging
import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from setup_support import SetupApp


class GNATCollReadline(SetupApp):
    name = 'gnatcoll_readline'
    project = 'gnatcoll_readline.gpr'
    description = 'GNATColl Readline bindings'

    def create(self):
        super(GNATCollReadline, self).create()
        self.build_cmd.add_argument(
            '--debug',
            help='build project in debug mode',
            action="store_true",
            default=False)
        self.build_cmd.add_argument(
            '--accept-gpl',
            help='accept the GPL license',
            action="store_true",
            default=False)

    def update_config(self, config, args):
        assert args.accept_gpl, "--accept-gpl is required"

        logging.info('%-26s %s',
                     'Libraries kind', ", ".join(config.data['library_types']))

        # Set library version
        with open(os.path.join(config.source_dir, '..',
                               'version_information'), 'r') as fd:
            version = fd.read().strip()
        config.set_data('GNATCOLL_VERSION', version, sub='gprbuild')

        # Set build mode
        config.set_data('BUILD', 'DEBUG' if args.debug else 'PROD',
                        sub='gprbuild')
        logging.info('%-26s %s', 'Build mode',
                     config.data['gprbuild']['BUILD'])

        # Set GNATCOLL_OS
        if 'darwin' in config.data['canonical_target']:
            gnatcoll_os = 'osx'
        elif 'windows' in config.data['canonical_target']:
            gnatcoll_os = 'windows'
        else:
            # Assume this is an Unix system
            gnatcoll_os = 'unix'
        config.set_data('GNATCOLL_OS', gnatcoll_os, sub='gprbuild')

    def variants(self, config, cmd):
        result = []
        for library_type in config.data['library_types']:
            gpr_vars = {'LIBRARY_TYPE': library_type,
                        'XMLADA_BUILD': library_type,
                        'GPR_BUILD': library_type}
            if cmd == 'install':
                result.append((['--build-name=%s' % library_type,
                                '--build-var=LIBRARY_TYPE'],
                               gpr_vars))
            else:
                result.append(([], gpr_vars))
        return result


if __name__ == '__main__':
    app = GNATCollReadline()
    sys.exit(app.run())
